package api

import (
	"assetcenter/db/model"
	"assetcenter/db/repo"
	"fmt"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

// 注册设计相关接口
func CreateDatabaseDesignRouter(router *GinRouter) {

	router.GETJWT("/design/list", DesignList)
	router.POSTJWT("/design/create", DesignCreate)
	router.POSTJWT("/design/update", DesignSave)
	router.POSTJWT("/design/delete/:id", DesignDelete)
	router.GETJWT("/design/detail/:id", DesignDetail)
	router.POSTJWT("/design/save", DesignSave)
}

func DesignList(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)

	if query == nil {
		query = map[string]interface{}{}
	}

	query["userId"], _ = primitive.ObjectIDFromHex(apictx.User.ID)

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionDesigns,
		Page:        page,
		Size:        size,
		Query:       query,
		Project:     []string{"name", "thumbnail", "createTime"},
		Sort:        bson.M{"createTime": -1},
	})
}

// 创建设计
func DesignCreate(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	body := &struct {
		Name string
		From string
		Id   string
	}{}

	err := c.ShouldBindJSON(body)
	if err != nil {
		return nil, NewError("参数解析错误")
	}
	if len(body.Name) < 1 {
		body.Name = "未定义"
	}

	//创建空设计
	d3d := &model.Design3d{
		Name:       body.Name,
		CreateTime: time.Now(),
		Products:   []*model.ProductHeader{},
		Scenes:     []*model.DesignScene{},
	}
	d3d.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, d3d)

	// if len(body.From) < 1 || len(body.Id) < 1 {
	// 	return nil, NewError("单品Id或类型不能为空!")
	// }
	// if body.From != "decorate" && body.From != "boxtpl" {
	// 	return nil, NewError("不支持的模型类型")
	// }

	// collection := UtilAssetCollectionName(body.From)
	// updateProjectIdCollection := ""
	// updateProjectId := ""

	// if body.From == "decorate" {
	// 	assetHeader := &model.AssetDecorateMesh{}
	// 	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{CollectName: collection, Query: repo.Map{"_id": body.Id}}, assetHeader)
	// 	if err != nil {
	// 		return nil, err
	// 	}
	// 	if !ok {
	// 		return nil, NewError("对应的装饰不存在!")
	// 	}

	// 	projectDecorate := &model.ProjectDecorateMesh{}
	// 	projectDecorate.CopyFromAsset(assetHeader)
	// 	//projectDecorate.ProjectId, _ = primitive.ObjectIDFromHex(prjId)
	// 	updateProjectIdCollection = repo.CollectionProjectDecorate

	// 	decorateId, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionProjectDecorate, projectDecorate)

	// 	if err != nil {
	// 		return nil, err
	// 	}
	// 	updateProjectId = decorateId

	// 	header := &model.ProductHeader{
	// 		Name:       projectDecorate.Name,
	// 		From:       body.From,
	// 		Id:         decorateId,
	// 		Thumbnail:  *projectDecorate.Thumbnail,
	// 		CreateTime: time.Now(),
	// 	}
	// 	project.Products = []*model.ProductHeader{header}
	// } else if body.From == "boxtpl" {
	// 	assetHeader := &model.Boxtpl{}
	// 	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{CollectName: collection, Query: repo.Map{"_id": body.Id}}, assetHeader)
	// 	if err != nil {
	// 		return nil, err
	// 	}
	// 	if !ok {
	// 		return nil, NewError("对应的装饰不存在!")
	// 	}

	// 	projectBoxtpl := &model.ProjectBoxtpl{}
	// 	projectBoxtpl.CopyFromAsset(assetHeader)
	// 	//projectDecorate.ProjectId, _ = primitive.ObjectIDFromHex(prjId)
	// 	updateProjectIdCollection = repo.CollectionProjectBoxTpl

	// 	boxId, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), updateProjectIdCollection, projectBoxtpl)
	// 	if err != nil {
	// 		return nil, err
	// 	}
	// 	updateProjectId = boxId

	// 	header := &model.ProductHeader{
	// 		Name:       projectBoxtpl.Name,
	// 		From:       body.From,
	// 		Id:         boxId,
	// 		Thumbnail:  *projectBoxtpl.Thumbnail,
	// 		CreateTime: time.Now(),
	// 	}
	// 	project.Products = []*model.ProductHeader{header}
	// }

	// prjId, err := repo.AddProject(apictx.CreateRepoCtx(), project)
	// if err != nil {
	// 	return nil, err
	// }

	// uid, _ := primitive.ObjectIDFromHex(prjId)
	// _, err = repo.RepoUpdateSetDocProps(apictx.CreateRepoCtx(), updateProjectIdCollection, updateProjectId, bson.M{"$set": bson.M{"projectId": uid}})
	// if err != nil {
	// 	repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionProject, prjId)
	// 	return nil, err
	// }
	// return prjId, nil
}

func DesignSave(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	body := &model.Design3d{}
	err := c.ShouldBindJSON(body)
	if err != nil {
		fmt.Println(err)
		return nil, NewError("参数解析错误" + err.Error())
	}
	if len(body.Id) < 1 {
		return nil, NewError("ID不能为空")
	}

	id := body.Id.Hex()
	body.Id = primitive.NilObjectID
	body.UserId = primitive.NilObjectID
	body.CreateTime = time.Time{}
	body.UpdateTime = time.Now()

	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, id, body)
}

func DesignDelete(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	if len(id) < 1 {
		return nil, NewError("参数不能为空")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionDesigns, id)
}

func DesignDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	if len(id) < 1 {
		return nil, NewError("参数id不能为空")
	}

	prj := &model.Design3d{}

	ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{CollectName: repo.CollectionDesigns, Query: repo.Map{"_id": id}}, prj)

	if err != nil {
		return nil, err
	}

	if !ok {
		return nil, NewError("没有数据!")
	}
	return prj, nil
}
