package api

import (
	"assetcenter/conf"
	"assetcenter/db"
	"assetcenter/db/model"
	"assetcenter/db/repo"
	"context"
	"errors"
	"fmt"
	"math"
	"time"

	"github.com/gin-gonic/gin"
)

func AssetImport(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	valueIdMap, err := ImportCategry(apictx.Svc.Mongo)

	if err != nil {
		return nil, err
	}

	err = ImportAssetCategory(apictx, valueIdMap)

	return true, err
}

func ImportCategry(dbMongo *db.MongoDB) (map[string]string, error) {

	valueIdMap := map[string]string{}

	CategoryConf := conf.AppConfig.Category
	if CategoryConf == nil {
		return valueIdMap, nil
	}

	if len(CategoryConf.DefaultCategory) < 1 {
		return valueIdMap, errors.New("category.defaultCategory 不能为空!")
	}

	ctx := &repo.RepoSession{
		Client: dbMongo,
		Ctx:    context.Background(),
	}

	userId := CategoryConf.QiyeUserId

	total, err := repo.RepoCountDoc(ctx, repo.CollectionCategories, repo.Map{})
	if err != nil {
		return valueIdMap, err
	}
	if total > 0 {
		return valueIdMap, errors.New("数据已存在，无法导入。请先清除现有数据")
	}

	var createOrderId = func(deep int32, brotherIndex int32, parentOrder int64) int64 {

		step := math.Pow(100, float64(4-deep))

		orderId := int64(step) * int64(brotherIndex+1)

		return int64(orderId + parentOrder)
	}

	var IteUpdate func(deep int32, cates []*conf.DefaultCategoryConf, parent string, parentOrder int64) error
	IteUpdate = func(deep int32, cates []*conf.DefaultCategoryConf, parent string, parentOrder int64) error {
		if len(cates) < 1 {
			return nil
		}
		for index, item := range cates {

			currOrderId := createOrderId(deep, int32(index), parentOrder)

			insertItem := &model.ResCategory{
				Type:       "imported",
				Name:       item.Name,
				Level:      &deep,
				Value:      item.Value,
				Parent:     parent,
				UserId:     userId,
				OrderId:    currOrderId,
				CreateTime: time.Now(),
			}

			id, err := repo.RepoAddDoc(ctx, repo.CollectionCategories, insertItem)
			if err != nil {
				return err
			}
			fmt.Println("added", id)
			valueIdMap[item.Value] = id
			err = IteUpdate(deep+1, item.Children, id, currOrderId)
			if err != nil {
				return err
			}
		}
		return nil
	}

	err = IteUpdate(0, CategoryConf.DefaultCategory, "", 0)
	if err != nil {
		return valueIdMap, err
	}
	return valueIdMap, nil
}

func ImportAssetCategory(apictx *ApiSession, valueIdMap map[string]string) error {

	AssetsConf := conf.AppConfig.Assets
	if AssetsConf == nil {
		return nil
	}

	if len(AssetsConf) < 1 {
		return errors.New("AppConfig.Assets 不能为空!")
	}

	userId := conf.AppConfig.Category.QiyeUserId

	for _, asset := range AssetsConf {
		if len(asset.DefaultCategory) > 0 {
			//添加默认分类设置

			ids := []string{}
			for _, cat := range asset.DefaultCategory {
				if len(valueIdMap[cat]) > 0 {
					ids = append(ids, valueIdMap[cat])
					continue
				}
				return errors.New(cat + "没有对应的分类ID")
			}
			assetCate := &model.LibCategory{Type: asset.Type, CreateTime: time.Now(), CategoryIds: ids, UserId: userId}
			id, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionDbAssetCategory, assetCate)
			if err != nil {
				return err
			}
			fmt.Sprintln("added "+asset.Type+"=>", id)
		}
	}
	return nil
}
