package api

import (
	"assetcenter/db/model"
	"assetcenter/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func CreateUserUploadMatRouter(router *GinRouter) {

	//上传2d面料
	router.POSTJWT("/upload/imgmat", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

		body := &struct {
			Name      string
			Category  string
			Category2 string
			Image     *model.OssType
		}{}
		c.ShouldBindJSON(&body)
		if body.Image == nil || len(body.Image.Url) < 1 {
			return nil, NewError("图片不能为空")
		}
		nopublic := false
		platform := true

		imgmat := &model.ImageMat{
			Category:   body.Category,
			Category2:  body.Category2,
			Name:       "2d面料",
			IsPublic:   &nopublic,
			Image:      body.Image,
			Platform:   &platform,
			CreateTime: time.Now(),
			State:      model.LibState_Created,
		}
		imgmat.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)
		if len(body.Name) > 0 {
			imgmat.Name = body.Name
		}
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionImageMat, imgmat)
	})

	//面料
	router.POSTJWT("/upload/fabric", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {

		body := &struct {
			Name      string
			Category  string
			Category2 string
			Thumbnail *model.OssType
		}{}

		c.ShouldBindJSON(&body)
		if body.Thumbnail == nil || len(body.Thumbnail.Url) < 1 {
			return nil, NewError("封面不能为空")
		}

		public := false
		fabric := &model.Fabric{
			Category:   body.Category,
			Category2:  body.Category2,
			ColorCards: []*model.MaterialHeader{},
			Name:       "面料1",
			Thumbnail:  &model.OssType{Url: "", Size: 0},
			CreateTime: time.Now(),
			IsPublic:   &public,
			Price:      0,
			State:      model.LibState_Created,
		}
		fabric.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)
		if len(body.Name) > 0 {
			fabric.Name = body.Name
		}
		return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionFabric, fabric)
	})

}
