package model

import (
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

type Matrix [16]float64

type SceneLight struct {
	Id        string  `bson:"id,omitempty" json:"id"`
	Type      string  `bson:"type,omitempty" json:"type"`
	Enable    bool    `bson:"enable,omitempty" json:"enable"`
	Matrix    Matrix  `bson:"matrix,omitempty" json:"matrix"`
	Color     Vect3   `bson:"color,omitempty" json:"color"`
	Intensity float64 `bson:"intensity,omitempty" json:"intensity"`
}

type DesignScene struct {
	Id         string    `bson:"id,omitempty"  json:"id"`
	Name       string    `bson:"name,omitempty"  json:"name"` //名字唯一
	CreateTime time.Time `bson:"createTime,omitempty"  json:"createTime"`
	Thumbnail  *OssType  `bson:"thumbnail,omitempty"  json:"thumbnail"`

	Product []*SceneProduct `bson:"products"  json:"products"`
	Env3d   *Env3dConfig    `bson:"env3d,omitempty"  json:"env3d"`
	Lights  []*SceneLight   `bson:"lights,omitempty"  json:"lights"`
}

type DesignProduct struct { //设计单品信息
	Id              primitive.ObjectID `bson:"_id,omitempty" json:"_id"`
	ProjectId       primitive.ObjectID `bson:"projectId,omitempty" json:"projectId"`
	FromDbConfId    string             `bson:"fromDbConfId,omitempty" json:"fromDbConfId"` //数据库定义Id
	FromAssetConfId string             `bson:"fromAssetConfId,omitempty" json:"fromAssetConfId"`
	FromId          string             `bson:"fromId,omitempty" json:"fromId"`
	Name            string             `bson:"name,omitempty"  json:"name"`
	Thumbnail       *OssType           `bson:"thumbnail,omitempty"  json:"thumbnail"`

	StaticMesh *StaticMeshSource `bson:"staticMesh,omitempty" json:"staticMesh"` //静态模型资产
	UpdateTime time.Time         `bson:"updateTime,omitempty"  json:"updateTime"`
}

type ProductHeader struct {
	Id              string    `bson:"id,omitempty"  json:"id"`
	FromDbConfId    string    `bson:"fromDbConfId,omitempty" json:"fromDbConfId"` //数据库定义Id
	FromAssetConfId string    `bson:"fromAssetConfId,omitempty" json:"fromAssetConfId"`
	FromId          string    `bson:"fromId,omitempty" json:"fromId"`
	Name            string    `bson:"name,omitempty"  json:"name"`
	Thumbnail       *OssType  `bson:"thumbnail,omitempty"  json:"thumbnail"`
	CreateTime      time.Time `bson:"createTime,omitempty"  json:"createTime"`
}

type Design3d struct {
	Id         primitive.ObjectID `bson:"_id,omitempty" json:"_id"`
	Name       string             `bson:"name,omitempty" json:"name"`
	Thumbnail  *OssType           `bson:"thumbnail" json:"thumbnail"`
	CreateTime time.Time          `bson:"createTime,omitempty" json:"createTime"`
	UpdateTime time.Time          `bson:"updateTime,omitempty" json:"updateTime"`
	UserId     primitive.ObjectID `bson:"userId,omitempty" json:"userId"`
	Products   []*ProductHeader   `bson:"products,omitempty" json:"products"`
	Scenes     []*DesignScene     `bson:"scenes,omitempty" json:"scenes"`
}
