package tree

import (
	"assetcenter/db/repo"
	funcgraph "assetcenter/funcGraph"
	"assetcenter/log"
	"fmt"

	"infish.cn/comm"
)

func ProcessPack(repeSession *repo.RepoSession, packId string, dbName string, coll string, natsUrl string) error {
	mesh := &comm.AssetPackage{}

	ok, err := repo.RepoSeachDoc(repeSession, &repo.DocSearchOptions{
		Db:          dbName,
		CollectName: coll,
		Query:       repo.Map{"_id": packId},
		Project:     []string{"config", "transTask", "source"},
	}, mesh)
	if err != nil {
		return err
	}
	if !ok {
		return fmt.Errorf("模型已删除")
	}

	transTask := mesh.TransTask

	if comm.TransTaskIsNotEmpty(transTask) {
		err = funcgraph.OsgConvRequest(transTask.File.Url, packId, dbName, coll, natsUrl)
	}
	return err
}

func CreatePackShadow(repeSession *repo.RepoSession, assetId string, dbName string, coll string, scale int, width int, natsUrl string) error {
	mesh := &comm.AssetPackage{}

	ok, err := repo.RepoSeachDoc(repeSession, &repo.DocSearchOptions{
		Db:          dbName,
		CollectName: coll,
		Query:       repo.Map{"_id": assetId},
		Project:     []string{"source", "config"},
	}, mesh)
	if err != nil {
		return err
	}
	if !ok {
		return fmt.Errorf("模型已删除")
	}

	meshUrl := ""

	if len(mesh.Config) > 1 {
		// 解析文件拿到file
		source := &comm.Queen3dPackageSource{}
		err := comm.FetchAndParseJSON(mesh.Config, source)
		if err != nil {
			log.Error("shadow parse json err: ", err)
			return err
		}
		mesh.Source = source
	}
	if mesh.Source != nil {
		if len(mesh.Source.Geoms) < 1 {
			return fmt.Errorf("数据模型异常")
		}
		geom := mesh.Source.Geoms[0]
		if geom.File != nil {
			meshUrl = geom.File.Url
		}
		if geom.Glb != nil && len(geom.Glb.Url) > 0 {
			meshUrl = geom.Glb.Url
		}
		if len(meshUrl) < 1 {
			return fmt.Errorf("数据模型异常")
		}
	}
	fmt.Println("commit func shadow : ", meshUrl, assetId, dbName, coll, natsUrl)

	err = funcgraph.CreateShadow(meshUrl, assetId, dbName, coll, int32(scale), int32(width), natsUrl)
	return err

}
