package tree

import (
	"assetcenter/db/repo"
	funcgraph "assetcenter/funcGraph"
	"assetcenter/log"
	"time"

	"infish.cn/comm"
)

func UploadAssetPackage(repoSession *repo.RepoSession, dbName string, Coll string, body *comm.AssetPackage, natsUrl string) (interface{}, error) {

	asset := body
	falseVale := false
	asset.Enable = &falseVale

	transTask := asset.TransTask
	shadowTask := asset.ShadowTask

	if comm.TransTaskIsNotEmpty(transTask) {
		// 转换文件存在,更新状态
		asset.Source = nil
	}

	asset.CreateTime = time.Now()
	asset.UpdateTime = time.Now()

	assetId, err := repo.RepoDbAddDoc(repoSession, dbName, Coll, asset)
	if err != nil {
		return nil, err
	}

	if comm.TransTaskIsNotEmpty(transTask) {
		// 模型转换
		funcgraph.OsgConvRequest(transTask.File.Url, assetId, dbName, Coll, natsUrl)

	}
	if comm.ShadowIsNotEmpty(shadowTask) {
		// 阴影转换
		funcgraph.CreateShadow(shadowTask.File.Url, assetId, dbName, Coll, 15, 512, natsUrl)
	}

	return assetId, nil
}

// tree.asset.update.header
func UpdateAssetPackageHeader(repoSession *repo.RepoSession, dbName string, Coll string, asset *comm.AssetPackage, natsUrl string) (interface{}, error) {

	log.Infof("%s,%s,%s\n", dbName, Coll, asset, natsUrl)
	log.Infof("%#v\n", *asset)

	transTask := asset.TransTask
	shadowTask := asset.ShadowTask

	assetId := asset.Id.Hex()
	_, err := repo.RepoUpdateSeDbDoc(repoSession, dbName, Coll, assetId, asset)

	if comm.TransTaskIsNotEmpty(transTask) {
		// 如果是大模型模式,在transTask中获取原模型文件url,并更新transTask
		funcgraph.OsgConvRequest(transTask.File.Url, assetId, dbName, Coll, natsUrl)
	}
	if comm.ShadowIsNotEmpty(shadowTask) {
		// 阴影转换
		funcgraph.CreateShadow(shadowTask.File.Url, assetId, dbName, Coll, 15, 512, natsUrl)
	}

	return nil, err
}
